@@ -52,6 +52,8 @@ class StationB(Station):
         metadata: Optional[dict] = None,
         num_samples: int = 96,
         pipette_max_volume: float = 200,
+        resuspend_bottom_height: float = 2,
+        resuspend_y_movement: float = 1,
         samples_per_col: int = 8,
         skip_delay: bool = False,
         supernatant_removal_air_gap: float = 5,
@@ -133,6 +135,8 @@ class StationB(Station):
         :param metadata: protocol metadata
         :param num_samples: The number of samples that will be loaded on the station B
         :param pipette_max_volume: The maximum volume in ul that the pipette (considering tip) can aspirate
+        :param resuspend_bottom_height: height at which aspirate and dispense take place to resuspend beads
+        :param resuspend_y_movement: side movement in + and - y direction of dispense to resuspend beads.
         :param samples_per_col: The number of samples in a column of the destination plate
         :param skip_delay: If True, pause instead of delay.
         :param supernatant_removal_air_gap: Air gap when removing the supernatant in uL
@@ -218,6 +222,8 @@ class StationB(Station):
         self._magheight_load = magheight_load
         self._magplate_model = magplate_model
         self._pipette_max_volume = pipette_max_volume
+        self._resuspend_bottom_height = resuspend_bottom_height
+        self._resuspend_y_movement = resuspend_y_movement
         self._supernatant_removal_air_gap = supernatant_removal_air_gap
         self._supernatant_removal_aspiration_rate = supernatant_removal_aspiration_rate
         self._supernatant_removal_aspiration_rate_first_phase = supernatant_removal_aspiration_rate_first_phase
@@ -341,6 +347,21 @@ class StationB(Station):
     def _tipracks(self) -> dict:
         return {"_tips300": "_m300",}
 
+    def set_magdeck(self, engaged: bool):
+        required_status = "engaged" if engaged else "disengaged"
+        self._ctx.comment("Check module status")
+        for j in range(3):
+            if self._magdeck.status == required_status:
+                self._ctx.comment("Status is {}".format(required_status))
+                break
+            self._ctx.comment("Try {} to set module to state {}".format(j+1, required_status))
+            if required_status == "engaged":
+                self._magdeck.engage(height=self._magheight)
+            else:
+                self._magdeck.disengage()
+        else:
+            raise Exception("Magdeck not able to be set to {}".format(required_status))
+
     def check(self):
         self._ctx.comment("Check module status")
         for j in range(3):
@@ -352,6 +373,7 @@ class StationB(Station):
             self.dual_pause("check module")
 
     def remove_supernatant(self, vol: float, stage: str = "remove supernatant"):
+        self.set_magdeck(True)
         num_trans, vol_per_trans = uniform_divide(vol-self._vol_last_trans,
                                                   self._pipette_max_volume - self._supernatant_removal_air_gap)
 
@@ -463,7 +485,9 @@ class StationB(Station):
             self._m300.flow_rate.dispense = self._wash_2_mix_dispense_rate
             self._m300.flow_rate.aspirate = self._wash_2_mix_aspiration_rate
             vertical_speed = self._wash_2_vertical_speed
-        self._magdeck.disengage()
+
+        self.set_magdeck(False)
+
         num_trans, vol_per_trans = uniform_divide(vol, self._wash_max_transfer_vol)
         
         for i, m in enumerate(self.mag_samples_m):
@@ -484,9 +508,11 @@ class StationB(Station):
                     self._m300.dispense(vol_per_trans, m.top())
                     self._m300.air_gap(self._wash_air_gap)
 
+                # Beads resuspension
                 # For mix_walk
-                a_locs = [m.bottom(self._bottom_headroom_height).move(
-                    Point(x=2 * (-1 if i % 2 else +1), y=2 * (2 * j / (mix_reps - 1) - 1))) for j in range(mix_reps)]
+                aspirate_loc = m.bottom(self._resuspend_bottom_height)
+                dispense_locs = [m.bottom(self._resuspend_bottom_height).move(Point(x=2 * (-1 if i % 2 else +1),
+                                                                                    y=j * self._resuspend_y_movement)) for j in [1, -1]]
                 # For mix
                 loc = m.bottom(self._bottom_headroom_height).move(Point(x=2 * (-1 if i % 2 else +1)))
 
@@ -497,14 +523,19 @@ class StationB(Station):
                                    speed=vertical_speed):
                     # Mix
                     if self._wash_mix_walk:
-                        mix_walk(self._m300, mix_reps, self._wash_mix_vol, a_locs, speed=self._wash_mix_speed, logger=self.logger)
+                        mix_walk(self._m300, mix_reps,
+                                 self._wash_mix_vol,
+                                 aspirate_loc,
+                                 dispense_locs,
+                                 speed=self._wash_mix_speed,
+                                 logger=self.logger)
                     else:
                         self._m300.mix(mix_reps, self._wash_mix_vol, loc)
                 self._m300.air_gap(self._wash_air_gap)
                 self.drop(self._m300)
 
-        self._magdeck.engage(height=self._magheight)
-        self.check()
+        self.set_magdeck(True)
+
         if self.run_stage("{} incubate".format(wash_name)):
             self.delay(self._wait_time_wash_on, self.get_msg_format("incubate on magdeck", self.get_msg("on")))
         self.remove_supernatant(vol, stage="remove {}".format(wash_name))
