@@ -1,5 +1,7 @@
+from opentrons.protocol_api.labware import Well
+
 from .b import StationB, labware_loader
-from typing import Tuple
+from typing import Tuple, List
 from opentrons.types import Point
 from ..utils import get_labware_json_from_filename, mix_bottom_top, WellWithVolume
 
@@ -28,9 +30,9 @@ class StationBTechnogenetics(StationB):
                  n_bottom: float = 3,
                  mix_incubate_on_time: float = 20,
                  mix_incubate_off_time: float = 5,
-                 postspin_incubation_time: float = 3,
+                 postspin_incubation_time: float = 0.5,
                  remove_wash_vol: float = 50,
-                 sample_mix_height: float = 0.3,
+                 sample_mix_height: float = 1,
                  sample_mix_times: int = 10,
                  sample_mix_vol: float = 180,
                  sample_vertical_speed: float = 35,
@@ -42,11 +44,13 @@ class StationBTechnogenetics(StationB):
                  tempdeck_temp: float = None,
                  tempdeck_auto_turnon: bool = False,
                  tempdeck_auto_turnoff: bool = True,
-                 thermomixer_incubation_time: float = 5,
+                 thermomixer_incubation_time: float = 0.5,
                  tipracks_slots: Tuple[str, ...] = ('4', '6', '7', '8', '9'),
                  wash_1_vol: float = 650,
                  wash_2_vol: float = 650,
                  wash_headroom: float = 1.05,
+                 wash_mix_vol: float = 180,
+                 wash_mix_walk: bool = True,
                  watchdog_serial_timeout_seconds: int = 30,
                  **kwargs
                  ):
@@ -86,6 +90,8 @@ class StationBTechnogenetics(StationB):
             wash_1_vol=wash_1_vol,
             wash_2_vol=wash_2_vol,
             wash_headroom=wash_headroom,
+            wash_mix_vol=wash_mix_vol,
+            wash_mix_walk=wash_mix_walk,
             **kwargs
         )
         self._beads_drying_time = beads_drying_time
@@ -155,14 +161,13 @@ class StationBTechnogenetics(StationB):
     def wash2(self):
         return self._elut12.wells()[:6]
 
-    
     @staticmethod
     def wash_getcol(sample_col_idx: int, wash_cols: int, source):
         return source[sample_col_idx // 2]
-    
-    def mix_samples(self):
-        for i, m in enumerate(self.temp_samples_m):
-            if self.run_stage("mix sample {}/{}".format(i + 1, len(self.mag_samples_m))):
+
+    def mix_samples(self,  wells: List[Well], stage_name: str = "mix sample"):
+        for i, m in enumerate(wells):
+            if self.run_stage("{} {}/{}".format(stage_name, i + 1, len(wells))):
                 self._m300.flow_rate.aspirate = self._mix_samples_rate
                 self._m300.flow_rate.dispense = self._mix_samples_rate
 
@@ -182,22 +187,34 @@ class StationBTechnogenetics(StationB):
                 self._m300.move_to(m.top(0), speed=self._sample_vertical_speed)
                 self._m300.air_gap(self._bind_air_gap)
                 self.drop(self._m300)
-    
+
+    def incubate_and_mix(self):
+        self.tempdeck_set_temperature(self._incubation_temperature)
+
+        self.delay_start_count()
+        self.mix_samples(self.temp_samples_m)
+
+        if self.run_stage("incubation"):
+            self.delay_wait_to_elapse(minutes=self._incubation_time)
+
+        self.tempdeck_deactivate()
+
     def elute(self, positions=None, transfer: bool = False, stage: str = "elute"):
         if positions is None:
             positions = self.temp_samples_m
-        self._magdeck.disengage()
+        self.set_magdeck(False)
         super(StationBTechnogenetics, self).elute(positions=positions, transfer=transfer, stage=stage)
-        self._magdeck.disengage()
     
     def remove_wash(self, vol, stage: str = "remove wash"):
         self.remove_supernatant(vol, stage)
-    
-    def final_transfer(self):
-        self._m300.flow_rate.aspirate = self._final_transfer_rate_aspirate
-        self._m300.flow_rate.dispense = self._final_transfer_rate_dispense
-        n = len(list(zip(self.mag_samples_m, self.pcr_samples_m)))
 
+    def final_transfer(self):
+        self.set_magdeck(True)
+        self.set_flow_rate(aspirate=self._final_transfer_rate_aspirate, dispense=self._final_transfer_rate_dispense)
+        self.final_transfer_movements()
+
+    def final_transfer_movements(self):
+        n = len(list(zip(self.mag_samples_m, self.pcr_samples_m)))
         for i, (m, e) in enumerate(zip(self.mag_samples_m, self.pcr_samples_m)):
             if self.run_stage("final transfer {}/{}".format(i + 1, n)):
                 self.pick_up(self._m300)
@@ -214,72 +231,62 @@ class StationBTechnogenetics(StationB):
                 self._m300.air_gap(self._elute_air_gap)
                 self.drop(self._m300)
 
-    def incubate_samples(self):
-        if self.run_stage("incubation"):
-            self.tempdeck_set_temperature(self._incubation_temperature)
-            self.delay(self._incubation_time)
-            self.tempdeck_deactivate()
+    def spin(self, stage_name: str):
+        if self.run_stage("spin deepwell {}".format(stage_name)):
+            self.set_magdeck(False)
+            self.dual_pause("spin the deepwell", between=self.set_external)
+            self.set_internal()
+
+    def second_removal(self, stage_name: str):
+        if self.run_stage("post spin incubation {}".format(stage_name)):
+            self.set_magdeck(True)
+            self.delay(self._postspin_incubation_time, self.get_msg_format("incubate on magdeck", self.get_msg("on")))
+        self.remove_wash(self._remove_wash_vol, "remove {} after spin".format(stage_name))
+
+    def spin_and_remove(self, stage_name: str):
+        self.spin(stage_name)
+        self.second_removal(stage_name)
+
+    def set_flow_rate(self, aspirate: float = None, dispense: float = None):
+        if aspirate is not None:
+            self._m300.flow_rate.aspirate = aspirate
+        if dispense is not None:
+            self._m300.flow_rate.dispense = dispense
 
     def body(self):
         self.logger.info(self.get_msg_format("volume", "wash 1", self._wash_headroom * self._wash_1_vol * self._num_samples / 1000))
         self.logger.info(self.get_msg_format("volume", "wash 2", self._wash_headroom * self._wash_2_vol * self._num_samples / 1000))
         self.logger.info(self.get_msg_format("volume", "elution buffer", self._wash_headroom * self._elution_vol * self._num_samples / 1000))
 
-        self.incubate_samples()
-        self.mix_samples()
+        self.incubate_and_mix()
 
-        self.pause("move plate to magdeck")
-
-        # if self.run_stage("mix incubate on"):
-        #     self.delay(self._mix_incubate_on_time, self.get_msg_format("incubate on magdeck", self.get_msg("off")))
-        self._magdeck.engage(height=self._magheight)
-        self.check()
         if self.run_stage("mix incubate off"):
+            self.pause("move plate to magdeck")
+            self.set_magdeck(True)
             self.delay(self._mix_incubate_off_time, self.get_msg_format("incubate on magdeck", self.get_msg("on")))
 
         self.remove_supernatant(self._starting_vol)
+
         self.wash(self._wash_1_vol, self.wash1, self._wash_1_times, "wash A")
 
-        self._magdeck.disengage()
+        self.spin("wash A")
 
-        if self.run_stage("spin deepwell wash A"):
-            self.dual_pause("spin the deepwell", between=self.set_external)
-            self.set_internal()
-            self.dual_pause("Add wash B and elute buffer in slot {}{}".format(
-                self.wash2[0].parent, " and {}".format(self.water.parent) if self.wash2[0].parent != self.water.parent else ""))
+        if self.run_stage("add wash B and elute"):
+            self.pause("Add wash B and elute buffer in slot {}{}".format(
+                self.wash2[0].parent,
+                " and {}".format(self.water.parent) if self.wash2[0].parent != self.water.parent else ""))
 
-        self._magdeck.engage(height=self._magheight)
-        self.check()
-
-        if self.run_stage("post spin incubation wash A"):
-            self.delay(self._postspin_incubation_time, self.get_msg_format("incubate on magdeck", self.get_msg("on")))
-
-        self.remove_wash(self._remove_wash_vol, "remove wash A after spin")
-
-        self._magdeck.disengage()
+        self.second_removal("wash A")
 
         self.wash(self._wash_2_vol, self.wash2, self._wash_2_times, "wash B")
-
-        self._magdeck.disengage()
-        if self.run_stage("spin deepwell wash B"):
-            self.dual_pause("spin the deepwell", between=self.set_external)
-            self.set_internal()
-
-        self._magdeck.engage(height=self._magheight)
-        self.check()
-
-        if self.run_stage("post spin incubation wash B"):
-            self.delay(self._postspin_incubation_time, self.get_msg_format("incubate on magdeck", self.get_msg("on")))
+        self.spin_and_remove("wash B")
 
         if self._tempdeck_temp is not None and not self._tempdeck_auto_turnon:
             # self._tempdeck.start_set_temperature(self._tempdeck_temp)
             self.tempdeck_set_temperature(self._tempdeck_temp)
 
-        self.remove_wash(self._remove_wash_vol, "remove wash B after spin")
-
-        self._magdeck.disengage()
-
         if self.run_stage("beads drying"):
+            self.set_magdeck(False)
             self.delay(self._beads_drying_time, msg="beads drying")
 
         self.elute(self.mag_samples_m)
@@ -288,17 +295,16 @@ class StationBTechnogenetics(StationB):
             self.dual_pause("seal the deepwell", between=self.set_external)
             self.set_internal()
 
-        self._magdeck.engage(height=self._magheight)
-        self.check()
-        if self.run_stage("post thermomixer incubation"):
-            self.delay(self._thermomixer_incubation_time, self.get_msg_format("incubate on magdeck", self.get_msg("on")))
-
         if self.run_stage("input PCR"):
-            self.dual_pause("input PCR")
+            self.pause("input PCR")
+
+        if self.run_stage("post thermomixer incubation"):
+            self.set_magdeck(True)
+            self.delay(self._thermomixer_incubation_time, self.get_msg_format("incubate on magdeck", self.get_msg("on")))
 
         self.final_transfer()
 
-        self._magdeck.disengage()
+        self.set_magdeck(False)
         self.logger.info(self.msg_format("move to PCR"))
 
     def tempdeck_set_temperature(self, temperature):
@@ -311,6 +317,39 @@ class StationBTechnogenetics(StationB):
         self._tempdeck.deactivate()
         self.watchdog_stop()
 
+
+class StationBTechnogeneticsSaliva(StationBTechnogenetics):
+    def __init__(self,
+                 incubation_heatup_time: float = 15,
+                 incubation_mixing_time: float = 10,
+                 incubation_mix_times: int = 2,
+                 *args,
+                 **kwargs):
+        """ Build a :py:class:`.StationBTechnogeneticsSaliva`.
+            :param incubation_heatup_time: minutes needed to warm up the deepwell plate to the incubation temperature (experimentally found)
+            :param incubation_mixing_time: minutes of incubation during mix
+            :param incubation_mix_times: number of mix during incubation
+        """
+        super(StationBTechnogeneticsSaliva, self).__init__(*args, **kwargs)
+        self._incubation_heatup_time = incubation_heatup_time
+        self._incubation_mixing_time = incubation_mixing_time
+        self._incubation_mix_times = incubation_mix_times
+
+    def incubate_and_mix(self):
+        self.tempdeck_set_temperature(self._incubation_temperature)
+
+        if self.run_stage("initial incubation"):
+            self.delay(self._incubation_heatup_time)
+
+        for i in range(self._incubation_mix_times):
+            self.delay_start_count()
+            self.mix_samples(self.temp_samples_m, "mix samples {}".format(i+1))
+            if self.run_stage("incubation after mix {}".format(i+1)):
+                self.delay_wait_to_elapse(minutes=self._incubation_mixing_time)
+
+        self.tempdeck_deactivate()
+
+
 if __name__ == "__main__":
     StationBTechnogenetics(metadata={'apiLevel': '2.3'}).simulate()
 
