@@ -1,6 +1,6 @@
 from ..station import Station, labware_loader, instrument_loader
 from ..multi_tube_source import MultiTubeSource
-from ..utils import uniform_divide, MoveWithSpeed
+from ..utils import uniform_divide, MoveWithSpeed, get_labware_json_from_filename
 import math
 import logging
 
@@ -12,11 +12,11 @@ class BioerProtocol(Station):
             pk_tube_bottom_height = 2,
             mm_tube_bottom_height = 2,
             pcr_bottom_headroom_height = 4.5,
-            dw_bottom_height = 13.5,
+            dw_bottom_height = 17.5,
             mix_bottom_height = 0.5,
-            mix_bottom_height_dw = -1.8,
+            mix_bottom_height_dw = 2.2,
             mm_plate_bottom_height = 1.2,
-            dw_elutes_bottom_height = -3,
+            dw_elutes_bottom_height = 0,
             pk_volume_tube = 320,
             vol_pk_offset = 5,
             vol_mm_offset = 10,
@@ -35,8 +35,10 @@ class BioerProtocol(Station):
             mastermix_phase: bool = False,
             vertical_offset = -16,
             slow_vertical_speed: float = 25,
+            very_slow_vertical_speed: float = 5,
             elution_air_gap = 10,
             final_mix_blow_out_height = -2,
+            p300_tip_max_volume: float = 200,        # ul max volume managed by tips
             ** kwargs):
 
         super(BioerProtocol, self).__init__(
@@ -75,9 +77,12 @@ class BioerProtocol(Station):
         self._vol_pk_offset = vol_pk_offset
         self._vol_mm_offset = vol_mm_offset
         self._slow_vertical_speed = slow_vertical_speed
+        self._very_slow_vertical_speed = very_slow_vertical_speed
         self._elution_air_gap = elution_air_gap
         self._final_mix_blow_out_height = final_mix_blow_out_height
         self._s300_fake_aspirate: bool = True
+        self._p300_fake_aspirate: bool = True
+        self._p300_tip_max_volume = p300_tip_max_volume
 
         if self._num_samples > 80:
             self._dws = ['8', '9', '5', '6', '2', '3']
@@ -109,9 +114,11 @@ class BioerProtocol(Station):
     def load_tube_block(self):
         self._tube_block = self._ctx.load_labware(self._tube_block_model, '4', 'screw tube aluminum block for proteinase and mastermix')
 
-    @labware_loader(5, "_dests_plate")
+    @labware_loader(5, "_dests_plates")
     def load_dests_plate(self):
-        self._dests_plates = [self._ctx.load_labware('nest_96_wellplate_2ml_deep', slot, '96-deepwell sample plate' + str(e + 1))
+        self._dests_plates = [self._ctx.load_labware_from_definition(
+                                    get_labware_json_from_filename("bioer_96_wellplate_2000ul.json"),
+                                    slot, 'Bioer 96-deepwell sample plate 1' + str(e + 1))
                                 for e, slot in enumerate(self._dws)]
 
     @instrument_loader(0, "_p300")
@@ -177,37 +184,37 @@ class BioerProtocol(Station):
     def beads_destinations(self):
         return [row for plate in self._dests_plates for row in plate.rows()[0]][5::6]
 
-
-    def transfer_proteinase(self):
+    def transfer_proteinase(self, stage="transfer proteinase"):
         done_samples = 0
         num_cycle = 1
         num_samples_per_fill = 16
 
-        self.logger.info("Trasferring proteinase from tube to deepwells")
+        if self.run_stage(stage):
+            self.logger.info("Trasferring proteinase from tube to deepwells")
 
-        self.pick_up(self._s300)
-        while done_samples < self._num_samples:
-            samples_to_do = self.proteinase_destinations_unique[done_samples:(done_samples + num_samples_per_fill)]
-            # self.logger.info("Cycle {} - samples to do: {}".format(num_cycle, samples_to_do))
-            # self.logger.info(
-            #     "PK:Cycle {} - before filling: samples done: {}, samples to do: {}".format(num_cycle, done_samples,
-            #                                                                                len(samples_to_do)))
-            if num_cycle > 1:
-                self._vol_pk_offset = 0
-            vol_pk = len(samples_to_do) * self._pk_volume
-            if num_cycle == 1:
-                self._s300.aspirate(self._vol_pk_offset, self._tube_block.wells()[0].bottom(self._pk_tube_bottom_height))
-            self._pk_tube_source.prepare_aspiration(vol_pk, self._pk_tube_bottom_height)
-            self._pk_tube_source.aspirate(self._s300)
-            #self.logger.info("Aspirating {} at: {} mm".format(vol_pk, self._pk_tube_bottom_height))
-            for s, ss in enumerate(samples_to_do):
-                self._s300.dispense(self._pk_volume, ss.bottom(self._dw_bottom_height))
-                self._s300.touch_tip(ss, 0.6, self._vertical_offset)
-            done_samples += num_samples_per_fill
-            # self.logger.info("PK:Cycle {} - after distribution: samples done: {}".format(num_cycle, done_samples))
-            num_cycle += 1
-        if self._s300.has_tip:
-            self.drop(self._s300)
+            self.pick_up(self._s300)
+            while done_samples < self._num_samples:
+                samples_to_do = self.proteinase_destinations_unique[done_samples:(done_samples + num_samples_per_fill)]
+                # self.logger.info("Cycle {} - samples to do: {}".format(num_cycle, samples_to_do))
+                # self.logger.info(
+                #     "PK:Cycle {} - before filling: samples done: {}, samples to do: {}".format(num_cycle, done_samples,
+                #                                                                                len(samples_to_do)))
+                if num_cycle > 1:
+                    self._vol_pk_offset = 0
+                vol_pk = len(samples_to_do) * self._pk_volume
+                if num_cycle == 1:
+                    self._s300.aspirate(self._vol_pk_offset, self._tube_block.wells()[0].bottom(self._pk_tube_bottom_height))
+                self._pk_tube_source.prepare_aspiration(vol_pk, self._pk_tube_bottom_height)
+                self._pk_tube_source.aspirate(self._s300)
+                #self.logger.info("Aspirating {} at: {} mm".format(vol_pk, self._pk_tube_bottom_height))
+                for s, ss in enumerate(samples_to_do):
+                    self._s300.dispense(self._pk_volume, ss.bottom(self._dw_bottom_height))
+                    self._s300.touch_tip(ss, 0.6, self._vertical_offset)
+                done_samples += num_samples_per_fill
+                # self.logger.info("PK:Cycle {} - after distribution: samples done: {}".format(num_cycle, done_samples))
+                num_cycle += 1
+            if self._s300.has_tip:
+                self.drop(self._s300)
 
 
     def mix_beads(self):
@@ -223,78 +230,92 @@ class BioerProtocol(Station):
         if self._p300.has_tip:
             self.drop(self._p300)
 
-
-    def transfer_mastermix(self):
-        done_samples = 0
-        num_cycle = 1
-        num_samples_per_fill = 8
-
-        samples_with_controls = self.sample_dests_wells
-        samples_with_controls += self.control_wells_not_in_samples
+    def transfer_mastermix(self, stage="transfer mastermix"):
+        samples_with_controls = self.sample_dests_wells + self.control_wells_not_in_samples
 
         self.logger.info("Trasferring mastermix from tube to pcr plate")
 
-        self.pick_up(self._s300)
+        for i, sample in enumerate(samples_with_controls):
+            remaining_samples = len(samples_with_controls) - i
 
-        while done_samples < self._num_samples + len(self.control_wells_not_in_samples):
-            samples_to_do = samples_with_controls[done_samples:(done_samples + num_samples_per_fill)]
-            # self.logger.info("Cycle {} - samples to do: {}".format(num_cycle, samples_to_do))
-            # self.logger.info("MM:Cycle {} - before filling: samples done: {}, samples to do: {}".format(num_cycle,
-            #                                                                     done_samples, len(samples_to_do)))
+            if self.run_stage("{} {}/{}".format(stage,  i + 1, len(samples_with_controls))):
+                if not self._s300.has_tip:
+                    self.pick_up(self._s300)
+
+                self.fake_aspirate_mmix()
+                self.check_and_aspirate_mmix(remaining_samples)
 
-            if self._s300_fake_aspirate:
-                self._s300_fake_aspirate = False
-                self._s300.aspirate(10, self._tube_block.wells()[0].top())
-                self._s300.dispense(10)
-            if num_cycle > 1:
-                self._vol_mm_offset = 0
-            vol_mm = (len(samples_to_do) * self._mm_volume) + self._vol_mm_offset
-            self._mm_tube_source.prepare_aspiration(vol_mm, min_height=self._mm_tube_bottom_height)
-            self._mm_tube_source.aspirate(self._s300)
-            # self.logger.info("Aspirating at: {} mm".format(self._mm_tube_bottom_height))
-            for s, ss in enumerate(samples_to_do):
                 with MoveWithSpeed(self._s300,
-                                   from_point=ss.bottom(self._mm_plate_bottom_height + 5),
-                                   to_point=ss.bottom(self._mm_plate_bottom_height),
+                                   from_point=sample.bottom(self._mm_plate_bottom_height + 5),
+                                   to_point=sample.bottom(self._mm_plate_bottom_height),
                                    speed=self._slow_vertical_speed, move_close=False):
                     self._s300.dispense(self._mm_volume)
-            done_samples += num_samples_per_fill
-            # self.logger.info("MM:Cycle {} - after distribution: samples done: {}".format(num_cycle, done_samples))
-            num_cycle += 1
+            else:
+                # Stage not executed
+                self._mm_tube_source.use_volume_only(self._mm_volume)
         if self._s300.has_tip:
             self.drop(self._s300)
 
+    def check_and_aspirate_mmix(self, num_samples):
+        '''This function will check if the tip has mastermix to dispense to one sample.
+            If the tip is empty it will aspirate the needed quantity to serve the num_samples passed
+            @param num_samples: num samples that needed to be served
+            @return: If the aspiration is needed '''
+        if self._s300.current_volume < self._mm_volume:
+            max_num_samples = (self._p300_tip_max_volume - self._vol_mm_offset) // self._mm_volume
+            num_samples = min(max_num_samples, num_samples)
 
-    def transfer_elutes(self):
+            vol_mm = self._vol_mm_offset if self._s300.current_volume < self._vol_mm_offset else 0
+            vol_mm += (num_samples * self._mm_volume)
+            self.logger.info("Calculating mastermix: {}ul; num samples: {}".format(vol_mm, num_samples))
+
+            self._mm_tube_source.prepare_aspiration(vol_mm, min_height=self._mm_tube_bottom_height)
+            self._mm_tube_source.aspirate(self._s300)
+
+    def fake_aspirate_mmix(self):
+        if self._s300_fake_aspirate:
+            self._s300_fake_aspirate = False
+            self._s300.aspirate(10, self._mm_tube_source.get_current_well().top())
+            self._s300.dispense(10)
+
+    def transfer_elutes(self, stage="transfer elutes"):
         set_of_source = math.ceil(self._num_samples / self._max_sample_per_dw)
         source_plate = [row for plate in self._dests_plates for row in plate.columns()[4::6]]
         dests_sample_elute = self._dest_plate_elute.columns()[:self.num_cols]
 
         if self._pause_between_mastermix_and_elutes:
-            self.dual_pause(self.get_msg_format("insert deepwell", ", ".join([str(d) for d in self._dests_plates])))
+            self.dual_pause(self.get_msg_format("insert deepwell", ", ".join([str(d) for d in self._dests_plates])), home=(False, False))
 
         self.logger.info("Trasferring elutes from deepwells to pcr plate")
 
         done_col = 0
-        for e in range(set_of_source):
-            source_el = source_plate[done_col:(done_col + len(source_plate))]
-            for s, d in zip(source_el, dests_sample_elute):
-                samples = []
-                #self.logger.info("s,d:{}\n{}".format(s, d))
-                if any([w in d for w in self.control_dests_wells]):
-                    # self.logger.info("Using single pipette")
-                    for i, (w, z) in enumerate(zip(s, d)):
-                        if z not in self.control_dests_wells:
-                            samples.append((w, z))
-                    pipette = self._s300
-                else:
-                    samples.append((s[0], d[0]))
-                    pipette = self._p300
+        source_el = source_plate[done_col:(done_col + len(source_plate))]
+        for i, (s, d) in enumerate(zip(source_el, dests_sample_elute)):
+            samples_and_pipette = []
+            if any([w in d for w in self.control_dests_wells]):
+                # self.logger.info("Using single pipette")
+                for w, z in zip(s, d):
+                    if z not in self.control_dests_wells:
+                        samples_and_pipette.append((w, z, self._s300))
+            else:
+                samples_and_pipette.append((s[0], d[0], self._p300))
 
-                for t, o in samples:
-                    #self.logger.info("samples:{}".format(samples))
+            for j, (t, o, pipette) in enumerate(samples_and_pipette):
+                if self.run_stage("{} {}{}".format(stage, i+1, " {}/{}".format(j+1, len(samples_and_pipette)) if len(samples_and_pipette)>1 else "")):
                     self.pick_up(pipette)
-                    pipette.aspirate(self._elution_volume, t.bottom(self._dw_elutes_bottom_height))
+                    if self._p300_fake_aspirate or self._s300_fake_aspirate:
+                        pipette.aspirate(1, t.top())
+                        pipette.dispense(1)
+                        if pipette == self._p300:
+                            self._p300_fake_aspirate = False
+                        elif pipette == self._s300:
+                            self._s300_fake_aspirate = False
+
+                    with MoveWithSpeed(pipette,
+                                       from_point=t.bottom(self._dw_elutes_bottom_height + 3),
+                                       to_point=t.bottom(self._dw_elutes_bottom_height),
+                                       speed=self._very_slow_vertical_speed, move_close=False):
+                        pipette.aspirate(self._elution_volume)
                     pipette.air_gap(self._elution_air_gap)
                     pipette.dispense(self._elution_air_gap, o.top())
                     pipette.dispense(self._elution_volume, o.bottom(self._pcr_bottom_headroom_height))
@@ -302,7 +323,7 @@ class BioerProtocol(Station):
                     pipette.blow_out(o.top(self._final_mix_blow_out_height))
                     pipette.air_gap(self._elution_air_gap)
                     self.drop(pipette)
-                done_col = done_col + len(source_plate)
+            done_col = done_col + len(source_plate)
 
     def body(self):
 
@@ -362,7 +383,6 @@ class BioerProtocol(Station):
         #transfer_proteinase
         if self.transfer_proteinase_phase:
             self.pause(pk_requirements, home=False)
-            self.stage = "Transfer proteinase"
             self.transfer_proteinase()
 
         #mix_beads
@@ -378,12 +398,10 @@ class BioerProtocol(Station):
         if self._mastermix_phase:
             self.pause(control_positions, home=False)
             self.pause(mmix_requirements, home=False)
-            self.stage = "Transfer mastermix"
             self.transfer_mastermix()
 
         #transfer elutes
         if self.transfer_elutes_phase:
-            self.stage = "Transfer elutes"
             self.transfer_elutes()
 
     def drop(self, pip):
