@@ -1,5 +1,6 @@
 import json
 import os
+import time
 
 from opentrons.protocol_api import ProtocolContext, InstrumentContext, PairedInstrumentContext
 from opentrons.protocol_api.labware import Well
@@ -83,7 +84,8 @@ def mix_bottom_top(pip,
                    pos: Callable[[float], Location],
                    bottom: float,
                    top: float,
-                   last_dispense_rate: float = None):
+                   last_dispense_rate: float = None,
+                   last_mix_volume: float = None):
     """Custom mixing procedure aspirating at the bottom and dispensing at the top
     :param pip: The pipette
     :param reps: Number of repeats
@@ -91,13 +93,18 @@ def mix_bottom_top(pip,
     :param pos: Method for getting the position
     :param bottom: Offset for the bottom position
     :param top: Offset for the top position
-    :param last_dispense_rate: Dispense rate for the last transfer to keep the tip clean"""
+    :param last_dispense_rate: Dispense rate for the last transfer to keep the tip clean
+    :param last_mix_volume: volume to mix the last time."""
 
     for i in range(reps):
-        if i+1 == reps and last_dispense_rate is not None:
-            pip.flow_rate.dispense = last_dispense_rate
-        pip.aspirate(vol, pos(bottom))
-        pip.dispense(vol, pos(top))
+        mix_vol = vol
+        if i+1 == reps:
+            if last_dispense_rate is not None:
+                pip.flow_rate.dispense = last_dispense_rate
+            if last_mix_volume is not None:
+                mix_vol = last_mix_volume
+        pip.aspirate(mix_vol, pos(bottom))
+        pip.dispense(mix_vol, pos(top))
 
 
 def mix_walk(
@@ -248,6 +255,32 @@ def get_labware_json_from_filename(filename: str = ""):
         return json.load(f)
 
 
+class DelayManager:
+    def __init__(self,
+                 logger: logging.Logger = logging.getLogger("DelayManager")):
+        self._logger = logger
+        self._start_time = 0
+        self._duration = 0
+
+    def start(self):
+        self._start_time = time.time()
+        self._duration = 0
+        self._logger.info("Entering; now is {}".format(time.strftime("%H:%M", time.localtime(self._start_time))))
+
+    def stop(self):
+        self._duration = (time.time() - self._start_time) / 60
+        self._logger.info("Exiting; actual duration is {:.2f} minutes".format(self._duration))
+
+    def get_remaining_delay(self, seconds: float = 0, minutes: float = 0) -> float:
+        total_time_minutes = seconds/60 + minutes
+        delay = (total_time_minutes - self._duration) if total_time_minutes > self._duration else 0
+        self._logger.info("Returning delay of {:.2f} minutes".format(delay))
+        return delay
+
+    @property
+    def start_time(self):
+        return self._start_time
+
 # Copyright (c) 2020 Covmatic.
 # Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 # The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
